scSyncTool = {
    syncStart: 0,
    syncCount: 0,
    syncLimit: 100,
    syncDelay: 10, // how long to wait between calls

    syncRunning: false,

    animation: null,

    start: function() {
        this.syncStart = 0;
        this.setSyncRunning(true);
        jQuery('.sc-sync-tool-status')
            .removeClass('sc-notice-error')
            .show();
        jQuery('.sc-sync-tool-output').show();
        jQuery('.sc-sync-tool-list').empty();
        jQuery('.sc-progress-bar')
            .removeClass('sc-progress-bar-animate-stop')
            .removeClass('sc-progress-bar-orange')
            .removeClass('sc-progress-bar-red');

        this.doNextPage();
    },

    doNextPage: function() {
        if (this.syncStart >= this.syncCount) {
            this.setSyncRunning(false);
            return;
        }

        var data = {
            "action": "sc_sync_tool",
            "sync_start": scSyncTool.syncStart,
            "sync_limit": scSyncTool.syncLimit,
        };

        jQuery.ajax({
            type: "POST",
            url: ajaxurl,
            data: data,
            dataType: 'json'
        })
            .done(function (data, textStatus, jqXHR) {
                console.debug(data);
                console.debug(jqXHR);
                scSyncTool.processData(data);
            })
            .fail(function (jqXHR, textStatus, errorThrown) {
                console.error(jqXHR);

                if (jqXHR.status == 429) {
                    scSyncTool.slowDown();
                    return;
                }

                scSyncTool.setSyncRunning(false);

                jQuery('.sc-sync-tool-status').addClass('sc-notice-error');

                jQuery('.sc-progress-bar')
                    .addClass('sc-progress-bar-red')
                    .addClass('sc-progress-bar-animate-stop')
                    .removeClass('sc-progress-bar-orange');
                jQuery('.sc-sync-tool-status-message').html(
                    "<b>Failed</b> Syncing Failed on records in group "
                    + (scSyncTool.syncStart > 0 ? scSyncTool.syncStart : 1)
                    + " to " + (scSyncTool.getSyncEnd())
                    + "<Br>" + textStatus
                    + "(" + jqXHR.status +")<br><br> <span class='dashicons dashicons-warning' style='padding-right: 15px;'></span><b>Please contact Supporting Cast for more help.</b>" );
            });
    },

    setSyncRunning: function(bool) {
        this.syncRunning = bool;
        jQuery('#sc-sync-start').attr('disabled', bool);
        this.updateProgressBar();
    },

    processData: function(batchData) {
        this.syncStart += this.syncLimit; // Do this first in case we need to rollback to an earlier start due to a throttle limit being hit
        for (var k in batchData) {
            console.debug(batchData[k]);
            var IDs = k.split('@', 2); // WordpressID @ index

            var icon = '<span class="dashicons dashicons-yes"></span>';
            var trClass = 'status-ok';

            if (batchData[k].status_code == 429) {
                this.syncStart = parseInt(IDs[1]);
                this.slowDown();
                return;
            }

            if (batchData[k].status_code != 200) {
                icon = '<span class="dashicons dashicons-no"></span>';
                trClass = 'status-error';
                jQuery('.sc-progress-bar').addClass('sc-progress-bar-red');
                // Keep going when other errors occur
            }

            jQuery('.sc-progress-bar')
                .removeClass('sc-progress-bar-orange')
                .removeClass('sc-progress-bar-red');

            var tr = this.buildOutputRow([
                icon,
                IDs[1],
                IDs[0],
                (batchData[k].data && batchData[k].data.id ? batchData[k].data.id : ''),
                this.buildStatusString(batchData[k]),
            ]);

            tr.className = trClass;

            jQuery('.sc-sync-tool-list').append(tr);
        }

        this.nextPage();
    },

    buildOutputRow(columns) {
        var tr = document.createElement('tr');

        for (var col of columns ) {
            var elm = document.createElement('td');
            elm.innerHTML = col;
            tr.appendChild(elm);
        }
        return tr;
    },

    buildStatusString(batchData, elm) {

        if (batchData.data.errors) {
            var html = "";
            for (var key in batchData.data.errors) {
                html += batchData.data.errors[key][0] + "<br>";
            }
            return html;
        }

        if (batchData.data.message) {
            return batchData.data.message;
        }

        var html = "<b>Plan ID:</b>" + batchData.data.plan_id
            + ", <b>Product IDs:</b> ";
        if (batchData.data
            && (batchData.data.product_ids instanceof Array)
        ) {
            html += batchData.data.product_ids.join(',');
        }

        return html;
    },

    nextPage() {
        this.updateProgressBar();

        if (this.syncStart >= this.syncCount) {
            this.syncStart = this.syncCount;
            this.setSyncRunning(false);
            return;
        }

        setTimeout(this.doNextPage, this.syncDelay * 1000);
    },

    slowDown() {
        this.nextPage();
        jQuery('.sc-progress-bar').addClass('sc-progress-bar-orange');
        this.syncDelay += Math.round(this.syncDelay * .2); // Increase by 20%
        this.syncLimit = Math.round(this.syncLimit * .9); // Decrease by 10%
        jQuery('.sc-sync-tool-status-message').html("<b>Rate limit reached</b> Slowing down and restarting at " + (this.syncStart + 1));

    },

    getSyncEnd() {
        var end = this.syncStart + this.syncLimit;
        if (this.syncLimit >= this.syncCount) {
            end = this.syncCount;
        }
        return end;
    },

    updateProgressBar() {

        perc = Math.round((this.syncStart / this.syncCount) * 100);

        var end = this.getSyncEnd();

        var statusMsg = "<b>Running</b>  Syncing from " + this.syncStart + " to " + end + " of " + this.syncCount + " Total (" + perc + "%)";

        if (perc >= 100) {
            perc = 100;
            statusMsg = "<b>Finished</b>  Processed " + this.syncCount + " Users";
            jQuery('.sc-progress-bar')
                .addClass('sc-progress-bar-animate-stop')
                .removeClass('sc-progress-bar-orange')
                .removeClass('sc-progress-bar-red');
        }

        jQuery('.sc-sync-tool-status-message').html(statusMsg);

        if (this.animation) {
            this.animation.cancel();
        }

        jQuery(".sc-progress-bar > span").stop();

        jQuery(".sc-progress-bar > span").animate({
            width: perc + '%',
            complete() {
                scSyncTool.slowProgressBar();
            },
        }, 1200);

    },

    slowProgressBar() {

        // Slowly animate to the next point so it looks like it's moving
        perc = Math.round(( (this.getSyncEnd()) / this.syncCount) * 100);

        if (perc >= 100) {
            perc = 95; // Don't appear to finish before the last request comes in
        }

        this.animation = jQuery(".sc-progress-bar > span").animate(
            {
                width: perc + '%'
            },
            20000 // 60 seconds to get to the next position
        );
    }


};

