<?php

namespace Supportingcast;

use Exception;
use Memberful_User_Mapping_Repository;
use Supportingcast\Woocommere\ProcessOrder;
use \WP_User;

class WPUserInterpreter {

	protected $wpUser;
	protected $source;

	public function __construct(WP_User $wpUser, string $source)
	{
		$this->wpUser = $wpUser;
		$this->source = $source;
	}

	public function getScExternalId(): string
	{
		$id = get_user_meta($this->wpUser->ID, 'sc_external_id', true);

		if ($id) {
			return $id;
		}

		$id = $this->source . '-' . $this->getId();

		update_user_meta($this->wpUser->ID, 'sc_external_id', $id);

		return $id;
	}

	public function getScPlanId(): int
	{
		$sc_plan_options = get_option('sc-plans');

		if ('wordpress' === $this->source) {
			return (int) $sc_plan_options[0];
		}

		if ('memberful' === $this->source && is_array($sc_plan_options)) {
			return (int) $sc_plan_options[$this->getMemberfulPlanId()];
		}

		throw new Exception("Sorry, either a user isn't logged in, or we can't find a Supporting Cast plan for the user.");
	}

	public function getScProductIds(): array
	{
		return $this->getWoocommerceProductIds();
	}

	private function getId(): int
	{
		$id = null;

		if ('wordpress' === $this->source) {
			$id = $this->wpUser->ID;

		}

		if ('memberful' === $this->source) {
			$id = $this->getMemberfulMemberId();
		}

		if ($id) {
			return $id;
		}

		throw new Exception('Sorry, we don\'t know what membership service is used.');
	}

	private function getMemberfulMemberId(): int
	{
		try {
			$memberfulMap = new Memberful_User_Mapping_Repository();
			$member       = $memberfulMap->find_member_user_is_mapped_to($this->wpUser);

			return $member['id_of_member_user_is_mapped_to'];
		} catch (Exception $e) {
			throw new Exception('Sorry, we could not find a membership for this user.');
		}
	}

	private function getMemberfulPlanId(): int
	{
		$plans = get_user_meta($this->wpUser->ID)['memberful_subscription'][0] ?? null;

		if (!$plans) {
			throw new Exception('Sorry, we could not find any subscription info for this user.');
		}

		$id = array_keys(unserialize($plans))[0] ?? null;

		if (!$id) {
			throw new Exception('Sorry, we could not find a plan for this user.');
		}

		return (int) $id;
	}

	private function getWoocommerceProductIds(): array
	{
		$productIDs = [];

		if (!SC_WOOCOMMERCE_INSTALLED) {
			return $productIDs;
		}

		$orders = wc_get_orders(['customer_id' => $this->wpUser->ID]);

		foreach($orders as $order) {
			/** @var \WC_Order $order */
			$scProductIDs = ProcessOrder::getScProductIdsFromOrder($order);
			$productIDs = $productIDs + $scProductIDs; // array_merge will loose the keys
		}

		return $productIDs;
	}
}